/*****************************************************
*
*	Control program for the PitSchuLight TV-Backlight
*	(c) Peter Schulten, Mlheim, Germany
*	peter_(at)_pitschu.de
*
*	Die unvernderte Wiedergabe und Verteilung dieses gesamten Sourcecodes
*	in beliebiger Form ist gestattet, sofern obiger Hinweis erhalten bleibt.
*
* 	Ich stelle diesen Sourcecode kostenlos zur Verfgung und biete daher weder
*	Support an noch garantiere ich fr seine Funktionsfhigkeit. Auerdem
*	bernehme ich keine Haftung fr die Folgen seiner Nutzung.

*	Der Sourcecode darf nur zu privaten Zwecken verwendet und modifiziert werden.
*	Darber hinaus gehende Verwendung bedarf meiner Zustimmung.
*
*	History
*	09.06.2013	pitschu		Start of work
*/

#include "stm32f4xx.h"
#include <stdio.h>
#include "string.h"

#include "hardware.h"
#include "AvrXSerialIo.h"
#include "ws2812.h"
#include "tvp5150_dcmi.h"
#include "stm32_ub_usb_cdc.h"

extern void IRdecoderInit(void);
extern volatile uint32_t system_time;


typedef enum {
	MS_NONE		= 0,
	MS_FARBTON,
	MS_CONTRAST,
	MS_BRIGHTNESS,
	MS_SATURATION,
	MS_TOP,
	MS_LEFT,
	MS_RIGHT,
	MS_HEIGHT,
	MS_ICONTROL,
	MS_IMAGE_WID,
	MS_IMAGE_HIG,
	MS_VIDEO_SRC
} mainStates_e;

// rgbImage is a scaled imgae of the raw video image blocks. It can be sized from 1x1 to 48x28
rgbIcontroller_t	rgbImage [2*LEDS_X + 2*LEDS_Y];
short				rgbImageWid = 48;
short				rgbImageHigh = 28;

unsigned char status1 = 0;

// factor for rgbImage integrators
#define MAX_ICONTROL	128

short 				factorI = 20;			// 128 = 1.0

short				videoOffCount = 5;		// when counted down to 0, video is lost

// the dynXYZ values build the dynamic border of the none black rgbSlots rectangle. They are permanently adapted to the
// picture content. Black or nearly black borders are cut off.
short				dynLeft 	= 0;
short				dynRight 	= SLOTS_X-1;
short				dynTop 		= 0;
short				dynBottom 	= SLOTS_Y-1;

typedef struct {
	unsigned long	rgbSum;
	unsigned long	cnt;
} dynRGBsum_t;

dynRGBsum_t			dynColumns[SLOTS_X];
dynRGBsum_t			dynRows[SLOTS_Y];

void computeI (rgbIcontroller_t *pid, uint8_t r, uint8_t g, uint8_t b);


//----------------------------------------------------------------------------------------------------------



int main(void)
{
	mainStates_e mainState = MS_NONE;

	SystemInit();
	SystemCoreClockUpdate();
	SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
	init_systick ();		// runs at 100Hz

	BoardHardwareInit ();
	DelayCountInit ();
	IRdecoderInit();

	// Init vom USB-OTG-Port als CDC-Device (Virtueller-ComPort)
	UB_USB_CDC_Init();

	USART3_Init (115200);
	write_str2Host("Hi there. This is pitschu\n\r");
	/* Initialize LEDs and User_Button on STM32F4-Discovery --------------------*/
	STM_EVAL_PBInit(BUTTON_USER, BUTTON_MODE_EXTI);

	STM_EVAL_LEDInit(LED_GRN);
	STM_EVAL_LEDInit(LED_ORN);
	STM_EVAL_LEDInit(LED_RED);
	STM_EVAL_LEDInit(LED_BLU);


	//-------------------------------------------------
	// http://www.mikrocontroller.net/topic/261021
#include "math.h"
// TEST only: check for FPU is present and working
	float f = 1.01f;
	CORE_SysTickEn();
	vu32 it = CORE_GetSysTick();
	float f2 = f * 2.29f;
	vu32 it2 = CORE_GetSysTick() - it;
	printf ("FPU test: Mult: %d\n", (int)it2);

	it = CORE_GetSysTick ();
	f2 = sinf (f);
	it2 = CORE_GetSysTick() - it;
	printf ("FPU test: Sinus: %d\n", (int)it2);
//-------------------------------------------------

	WS2812init();				// init IO for RGB leds
	WS2812test();

	I2C_InitHardware();				// init I2C bus to TVP5150
	TVP5150init();				// init DCMI interface and start TVP5150
	delay_ms(100);
	TVP5150startCapture();

	while (1)
	{
		int i;
		int16_t c;

		while (captureReady == 0)		// time to handle input
		{
			if (system_time > 50)		// check every 500ms
			{
				system_time = 0;

				unsigned char s = TVP5150getStatus1 ();
				if (s != status1)
				{
					printf("\nVideo status changed: %02X at source %d, mode = %d\n",
							(int)s, (int)videoCurrentSource, (int)videoSourceSelect);
					status1 = s;
				}

				if (TVP5150hasVideoSignal() == 0)
				{
					if (videoOffCount > 0)
					{
						if (videoOffCount == 1)
						{
							printf("\nNo video signal at source %d, mode = %d\n", (int)videoCurrentSource, (int)videoSourceSelect);
							for (i = 0; i < WS2812_NR_LEDS; i++)		// blank all leds
							{
								ws2812ledRGB[i].R = 0;
								ws2812ledRGB[i].G = 0;
								ws2812ledRGB[i].B = 0;
							}

							if (videoSourceSelect == 0)			// no video connected -> switch source when suto select mode
							{
								short newSource = (videoCurrentSource == 1 ? 2 : 1);

								printf("\nAuto switch source to %d\n", (int)newSource);
								TVP5150selectVideoSource(newSource);
								// set green channel scan indicator
								ws2812ledRGB[newSource == 1 ? 0 : LEDS_Y+LEDS_X+LEDS_Y-1].G = 0x20;

								videoOffCount = 5;				// start search
							}
						}
						videoOffCount--;
					}
				}
				else
					videoOffCount = 5;
			}
#ifdef USE_USB
			if(UB_USB_CDC_GetStatus()==USB_CDC_CONNECTED)
				c = -1;
			else
				c = UB_VCP_CharRx ();
#else
			c = get_cHost();
#endif
			if (c > 0)				// got char from user
			{
				switch (c)
				{
				case 'f':
				case 'F':
					mainState = MS_FARBTON;
					printf("\nCurrent Hue control is %d\n", Hue_control);
					break;
				case 's':
				case 'S':
					mainState = MS_SATURATION;
					printf("\nCurrent Color_saturation is %d\n", Color_saturation);
					break;
				case 'b':
				case 'B':
					mainState = MS_BRIGHTNESS;
					printf("\nCurrent Brightness is %d\n", Brightness);
					break;
				case 'c':
				case 'C':
					mainState = MS_CONTRAST;
					printf("\nContrast Contrast is %d\n", Contrast);
					break;

				case 'l':
				case 'L':
					mainState = MS_LEFT;
					printf("\nCrop left is %d\n", (int)cropLeft/2);
					printf("Capture right is %d\n", (int)(cropLeft/2 + captureWidth));
					break;
				case 'r':
				case 'R':
					mainState = MS_RIGHT;
					printf("\nCrop left is %d\n", (int)cropLeft/2);
					printf("Capture right is %d\n", (int)(cropLeft/2 + captureWidth));
					break;
				case 't':
				case 'T':
					mainState = MS_TOP;
					printf("\nCrop top is %d\n", (int)cropTop);
					printf("Crop bottom is %d\n", (int)(cropTop+cropHeight-1));
					break;
				case 'h':
				case 'H':
					mainState = MS_HEIGHT;
					printf("\nCrop top is %d\n", (int)cropTop);
					printf("Crop bottom is %d\n", (int)(cropTop+cropHeight-1));
					break;

				case 'i':
				case 'I':
					mainState = MS_ICONTROL;
					printf("\nI-Control is %d\n", (int)factorI);
					break;

				case 'x':
				case 'X':
					mainState = MS_IMAGE_WID;
					printf("\nImage width in blocks is %d\n", (int)rgbImageWid);
					break;

				case 'y':
				case 'Y':
					mainState = MS_IMAGE_HIG;
					printf("\nImage height in blocks is %d\n", (int)rgbImageHigh);
					break;

				case 'v':
				case 'V':
					mainState = MS_VIDEO_SRC;
					printf("\nCurrent video source is %d, mode = %d\n", (int)videoCurrentSource, (int)videoSourceSelect);
					break;

				case '+':
				case '-':
				case 'd':
					switch (mainState)
					{
					case MS_FARBTON:
						if (c=='+' && Hue_control < 125) Hue_control += 1;
						if (c=='-' && Hue_control > -127) Hue_control -= 1;
						if (c=='d')	Hue_control = 0;
						printf("Hue control is %d\n", Hue_control);
						break;
					case MS_BRIGHTNESS:
						if (c=='+' && Brightness < 255) Brightness += 1;
						if (c=='-' && Brightness > 1) Brightness -= 1;
						if (c=='d')	Brightness = 128;
						printf("Brightness is %d\n", Brightness);
						break;
					case MS_SATURATION:
						if (c=='+' && Color_saturation < 255) Color_saturation += 1;
						if (c=='-' && Color_saturation > 1) Color_saturation -= 1;
						if (c=='d')	Color_saturation = 128;
						printf("Color_saturation is %d\n", Color_saturation);
						break;
					case MS_CONTRAST:
						if (c=='+' && Contrast < 197) Contrast += 1;
						if (c=='-' && Contrast > 1) Contrast -= 1;
						if (c=='d')	Contrast = 128;
						printf("Contrast is %d\n", Contrast);
						break;

					case MS_ICONTROL:
						if (c=='+' && factorI < MAX_ICONTROL) factorI += 1;
						if (c=='-' && factorI > 1) factorI -= 1;
						if (c=='d')	factorI = MAX_ICONTROL;
						printf("I-Control is %d\n", factorI);
						break;

					case MS_IMAGE_WID:
						if (c=='+' && rgbImageWid < SLOTS_X) rgbImageWid += 1;
						if (c=='-' && rgbImageWid > 1) rgbImageWid -= 1;
						if (c=='d')	rgbImageWid = SLOTS_X;
						printf("Image width in blocks is %d\n", rgbImageWid);
						memset((void*)&rgbImage[0], 0, sizeof (rgbImage));
						break;

					case MS_IMAGE_HIG:
						if (c=='+' && rgbImageHigh < SLOTS_Y) rgbImageHigh += 1;
						if (c=='-' && rgbImageHigh > 1) rgbImageHigh -= 1;
						if (c=='d')	rgbImageHigh = SLOTS_Y;
						memset((void*)&rgbImage[0], 0, sizeof (rgbImage));
						printf("Image height in blocks is %d\n", rgbImageHigh);
						break;

					case MS_LEFT:
						if (c=='+' && cropLeft/2 < 200 && (cropLeft/2 + captureWidth) < 840)
						{
							cropLeft += 8;
							captureWidth -= 4;
						}
						if (c=='-' && cropLeft/2 > 40 && (cropLeft/2 + captureWidth) < 840)
						{
							cropLeft -= 8;
							captureWidth += 4;
						}
						if (c=='d')	cropLeft = 168;
						printf("\nCrop left is %d\n", (int)cropLeft/2);
						printf("Capture right is %d\n", (int)(cropLeft/2 + captureWidth));
						memset((void*)&rgbSlots[0][0], 0, sizeof (rgbSlots));
						break;
					case MS_RIGHT:
						if (c=='+' && captureWidth < 800 && (cropLeft/2 + captureWidth) < 840)
						{
							captureWidth += 4;
						}
						if (c=='-' && captureWidth > 200)
						{
							captureWidth -= 4;
						}
						if (c=='d')	captureWidth = 684;
						printf("\nCrop left is %d\n", (int)cropLeft/2);
						printf("Capture right is %d\n", (int)(cropLeft/2 + captureWidth));
						memset((void*)&rgbSlots[0][0], 0, sizeof (rgbSlots));
						break;
					case MS_TOP:
						if (c=='+' && cropTop < 150 && (cropTop + cropHeight) < 306)
						{
							cropTop += 1;
							cropHeight -= 1;
						}
						if (c=='-' && cropTop > 4)
						{
							cropTop -= 1;
							cropHeight += 1;
						}
						if (c=='d')	cropTop = 16;
						printf("\nCrop top is %d\n", (int)cropTop);
						printf("Crop bottom is %d\n", (int)(cropTop+cropHeight-1));
						memset((void*)&rgbSlots[0][0], 0, sizeof (rgbSlots));
						break;
					case MS_HEIGHT:
						if (c=='+' && (cropTop + cropHeight) < 306)
						{
							cropHeight += 1;
						}
						if (c=='-' && cropHeight > 40)
						{
							cropHeight -= 1;
						}
						if (c=='d')	cropHeight = 288;
						printf("\nCrop top is %d\n", (int)cropTop);
						printf("Crop bottom is %d\n", (int)(cropTop+cropHeight-1));
						memset((void*)&rgbSlots[0][0], 0, sizeof (rgbSlots));
						break;

					default:
						break;
					}
					TVP5150setPictureParams();
					break;

					case '0':
					case '1':
					case '2':
						if (mainState == MS_VIDEO_SRC)
						{
							int s = c - '0';

							if (s == 0)
							{
								unsigned char i = TVP5150hasVideoSignal();

								if (i == 0 || videoSourceSelect != s)		// no signal -> switch source
								{
									TVP5150selectVideoSource(videoCurrentSource == 1 ? 2 : 1);
								}
							}
							else
							{
								TVP5150selectVideoSource(s);
							}

							videoSourceSelect = s;

							printf("\nCurrent video source is %d, mode = %d\n", (int)videoCurrentSource, (int)videoSourceSelect);
							delay_ms(1000);
							printf("    video signal state %d\n", (int)TVP5150hasVideoSignal());
							videoOffCount = 5;				// start search
						}
						break;

					default:
						printf("Usage: use +/- keys to set val; d=default\n");
						printf("     F=Hue, S=Saturation, B=Brightness, C=Contrast\n");
						printf("     L=Left, W=Width, T=Top, H=Height\n");
						printf("     I=I-factor of integrator (128 = MAX)\n");
						printf("     X=virtual image width in blocks\n");
						printf("     Y=virtual image height in blocks\n");
						printf("     V=select video source (1 or 2)\n");
						break;
				}
			}		// if (c > 0)
		}			// while captureReady == 0

		STM_EVAL_LEDOn(LED_BLU);

		/*
		 * Transform raw RGB values to scaled image
		 */
		if (1 == 1)				// on/off switch for testing
		{
			long  rVal, gVal, bVal;
			short curImage;
			short ledIdx;
			short cntVal;
			short dv;

			it = CORE_GetSysTick ();
			/*
			 * right edge
			 */
			curImage = 0;			// = bottom right
			cntVal = 0;
			rVal = gVal = bVal = 0;
			dv = 0;

			for (i = SLOTS_Y-1; i >= 0; i--)
			{
				rVal += 50 * (long)rgbSlots[i][dynRight].R + 30 * (long)rgbSlots[i][dynRight-1].R +
						15 * (long)rgbSlots[i][dynRight-2].R + 5 * (long)rgbSlots[i][dynRight-3].R;
				gVal += 50 * (long)rgbSlots[i][dynRight].G + 30 * (long)rgbSlots[i][dynRight-1].G +
						15 * (long)rgbSlots[i][dynRight-2].G + 5 * (long)rgbSlots[i][dynRight-3].G;
				bVal += 50 * (long)rgbSlots[i][dynRight].B + 30 * (long)rgbSlots[i][dynRight-1].B +
						15 * (long)rgbSlots[i][dynRight-2].B + 5 * (long)rgbSlots[i][dynRight-3].B;
				cntVal++;

				dv += rgbImageHigh;
				if (dv >= SLOTS_Y)
				{
					while (dv >= SLOTS_Y)
					{
						dv -= SLOTS_Y;
						computeI (&rgbImage [curImage], rVal / 100 / cntVal, gVal / 100 / cntVal, bVal / 100 / cntVal);
						curImage++;
					}
					rVal = gVal = bVal = 0;
					cntVal = 0;
				}
			}

			// left edge
			curImage = rgbImageHigh + rgbImageWid;			// top left starts at <right edge leds> + <top edge leds>
			cntVal = 0;
			rVal = gVal = bVal = 0;
			dv = 0;

			for (i = 0; i < SLOTS_Y; i++)
			{
				rVal += 50 * (long)rgbSlots[i][dynLeft].R + 30 * (long)rgbSlots[i][dynLeft+1].R +
						15 * (long)rgbSlots[i][dynLeft+2].R + 5 * (long)rgbSlots[i][dynLeft+3].R;
				gVal += 50 * (long)rgbSlots[i][dynLeft].G + 30 * (long)rgbSlots[i][dynLeft+1].G +
						15 * (long)rgbSlots[i][dynLeft+2].G + 5 * (long)rgbSlots[i][dynLeft+3].G;
				bVal += 50 * (long)rgbSlots[i][dynLeft].B + 30 * (long)rgbSlots[i][dynLeft+1].B +
						15 * (long)rgbSlots[i][dynLeft+2].B + 5 * (long)rgbSlots[i][dynLeft+3].B;
				cntVal++;

				dv += rgbImageHigh;
				if (dv >= SLOTS_Y)
				{
					while (dv >= SLOTS_Y)
					{
						dv -= SLOTS_Y;
						computeI (&rgbImage [curImage], rVal / 100 / cntVal, gVal / 100 / cntVal, bVal / 100 / cntVal);
						curImage++;
					}
					rVal = gVal = bVal = 0;
					cntVal = 0;
				}
			}

			// top edge
			curImage = rgbImageHigh;			// top right starts at <right edge leds>
			cntVal = 0;
			rVal = gVal = bVal = 0;
			dv = 0;

			for (i = SLOTS_X-1; i >= 0; i--)
			{
				rVal += 50 * (long)rgbSlots[dynTop][i].R + 30 * (long)rgbSlots[dynTop+1][i].R +
						15 * (long)rgbSlots[dynTop+2][i].R + 5 * (long)rgbSlots[dynTop+3][i].R;
				gVal += 50 * (long)rgbSlots[dynTop][i].G + 30 * (long)rgbSlots[dynTop+1][i].G +
						15 * (long)rgbSlots[dynTop+2][i].G + 5 * (long)rgbSlots[dynTop+3][i].G;
				bVal += 50 * (long)rgbSlots[dynTop][i].B + 30 * (long)rgbSlots[dynTop+1][i].B +
						15 * (long)rgbSlots[dynTop+2][i].B + 5 * (long)rgbSlots[dynTop+3][i].B;
				cntVal++;

				dv += rgbImageWid;
				if (dv >= SLOTS_X)
				{
					while (dv >= SLOTS_X)
					{
						dv -= SLOTS_X;
						computeI (&rgbImage [curImage], rVal / 100 / cntVal, gVal / 100 / cntVal, bVal / 100 / cntVal);
						curImage++;
					}
					rVal = gVal = bVal = 0;
					cntVal = 0;
				}
			}

			// bottom edge
			curImage = 2*rgbImageHigh + rgbImageWid;			// bottom left starts at <2*Y leds> + <top leds>
			cntVal = 0;
			rVal = gVal = bVal = 0;
			dv = 0;

			for (i = 0; i < SLOTS_X; i++)
			{
				rVal += 50 * (long)rgbSlots[dynBottom][i].R + 30 * (long)rgbSlots[dynBottom-1][i].R +
						15 * (long)rgbSlots[dynBottom-2][i].R + 5 * (long)rgbSlots[dynBottom-3][i].R;
				gVal += 50 * (long)rgbSlots[dynBottom][i].G + 30 * (long)rgbSlots[dynBottom-1][i].G +
						15 * (long)rgbSlots[dynBottom-2][i].G + 5 * (long)rgbSlots[dynBottom-3][i].G;
				bVal += 50 * (long)rgbSlots[dynBottom][i].B + 30 * (long)rgbSlots[dynBottom-1][i].B +
						15 * (long)rgbSlots[dynBottom-2][i].B + 5 * (long)rgbSlots[dynBottom-3][i].B;
				cntVal++;

				dv += rgbImageWid;
				if (dv >= SLOTS_X)
				{
					while (dv >= SLOTS_X)
					{
						dv -= SLOTS_X;
						computeI (&rgbImage [curImage], rVal / 100 / cntVal, gVal / 100 / cntVal, bVal / 100 / cntVal);
						curImage++;
					}
					rVal = gVal = bVal = 0;
					cntVal = 0;
				}
			}
			//-----------------------------------------------------------------------------------------------------------------

			STM_EVAL_LEDToggle (LED_BLU);

			captureReady = 0;

			if (videoOffCount >= 5)			// we have a good video signal
			{
				// distribute the scaled virtual image to the led buffer
				ledIdx = 0;			// = bottom right
				cntVal = 0;
				rVal = gVal = bVal = 0;
				dv = 0;

				for (i = 0; i < rgbImageHigh; i++)
				{
					rVal += rgbImage[i].R;
					gVal += rgbImage[i].G;
					bVal += rgbImage[i].B;
					cntVal++;

					dv += LEDS_Y;
					if (dv >= rgbImageHigh)
					{
						while (dv >= rgbImageHigh)
						{
							dv -= rgbImageHigh;
							ws2812ledRGB[ledIdx].R = rVal / cntVal;
							ws2812ledRGB[ledIdx].G = gVal / cntVal;
							ws2812ledRGB[ledIdx].B = bVal / cntVal;
							ledIdx++;
						}
						rVal = gVal = bVal = 0;
						cntVal = 0;
					}
				}

				ledIdx = LEDS_Y;			// = top right
				cntVal = 0;
				rVal = gVal = bVal = 0;
				dv = 0;

				for (i = rgbImageHigh; i < rgbImageWid + rgbImageHigh; i++)
				{
					rVal += rgbImage[i].R;
					gVal += rgbImage[i].G;
					bVal += rgbImage[i].B;
					cntVal++;

					dv += LEDS_X;
					if (dv >= rgbImageWid)
					{
						while (dv >= rgbImageWid)
						{
							dv -= rgbImageWid;
							ws2812ledRGB[ledIdx].R = rVal / cntVal;
							ws2812ledRGB[ledIdx].G = gVal / cntVal;
							ws2812ledRGB[ledIdx].B = bVal / cntVal;
							ledIdx++;
						}
						rVal = gVal = bVal = 0;
						cntVal = 0;
					}
				}

				ledIdx = LEDS_Y + LEDS_X;			// = top left
				cntVal = 0;
				rVal = gVal = bVal = 0;
				dv = 0;

				for (i = rgbImageWid + rgbImageHigh; i < rgbImageHigh + rgbImageWid + rgbImageHigh; i++)
				{
					rVal += rgbImage[i].R;
					gVal += rgbImage[i].G;
					bVal += rgbImage[i].B;
					cntVal++;

					dv += LEDS_Y;
					if (dv >= rgbImageHigh)
					{
						while (dv >= rgbImageHigh)
						{
							dv -= rgbImageHigh;
							ws2812ledRGB[ledIdx].R = rVal / cntVal;
							ws2812ledRGB[ledIdx].G = gVal / cntVal;
							ws2812ledRGB[ledIdx].B = bVal / cntVal;
							ledIdx++;
						}
						rVal = gVal = bVal = 0;
						cntVal = 0;
					}
				}

				ledIdx = LEDS_Y + LEDS_X + LEDS_Y;			// = bottom left
				cntVal = 0;
				rVal = gVal = bVal = 0;
				dv = 0;

				for (i = rgbImageHigh + rgbImageWid + rgbImageHigh; i < rgbImageHigh + rgbImageWid + rgbImageHigh + rgbImageWid; i++)
				{
					rVal += rgbImage[i].R;
					gVal += rgbImage[i].G;
					bVal += rgbImage[i].B;
					cntVal++;

					dv += LEDS_X;
					if (dv >= rgbImageWid)
					{
						while (dv >= rgbImageWid)
						{
							dv -= rgbImageWid;
							ws2812ledRGB[ledIdx].R = rVal / cntVal;
							ws2812ledRGB[ledIdx].G = gVal / cntVal;
							ws2812ledRGB[ledIdx].B = bVal / cntVal;
							ledIdx++;
						}
						rVal = gVal = bVal = 0;
						cntVal = 0;
					}
				}

			}		// transformation block

		}

		it2 = CORE_GetSysTick() - it;

		STM_EVAL_LEDToggle (LED_BLU);

		while (ledBusy)				// wait until last DMA is ready
			;
		WS2812update();
		STM_EVAL_LEDToggle (LED_BLU);
		STM_EVAL_LEDToggle (LED_BLU);
		STM_EVAL_LEDToggle (LED_BLU);
	}

}




void computeI (rgbIcontroller_t *pid, uint8_t r, uint8_t g, uint8_t b)
{
	long err;
	short i;

	err = (r - pid->R) * factorI;
	pid->Ri += err;
	i = pid->Ri / MAX_ICONTROL;
	if (i < 0) i = 0;
	if (i > 255) i = 255;
	pid->R = i;

	err = (g - pid->G) * factorI;
	pid->Gi += err;
	i = pid->Gi / MAX_ICONTROL;
	if (i < 0) i = 0;
	if (i > 255) i = 255;
	pid->G = i;

	err = (b - pid->B) * factorI;
	pid->Bi += err;
	i = pid->Bi / MAX_ICONTROL;
	if (i < 0) i = 0;
	if (i > 255) i = 255;
	pid->B = i;
}




#ifdef  USE_FULL_ASSERT

/**
 * @brief  Reports the name of the source file and the source line number
 *   where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @retval None
 */
void assert_failed(uint8_t* file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	printf("Wrong parameters value: file %s on line %d\r\n", file, (int)line);
}
#endif
